@@ -9,9 +9,14 @@ from functools import partial, update_wrapper
 from inspect import signature, BoundArguments
 from itertools import chain
 
+from undefined import undefined
 from .placeholder import _
 
 
+def _exist_placeholder(args):
+    return (a is _ or a is undefined for a in args)
+
+
 class ppartial(partial):
 
     def __new__(cls, func: Callable, /, *args, **kwargs):
@@ -24,14 +29,14 @@ class ppartial(partial):
     def __call__(self, /, *args, **kwargs):
         args_, kwargs_ = self.args, self.keywords
 
-        if _ in args_:
+        if _exist_placeholder(args_):
             args_it = iter(args)
-            pargs = (*(next(args_it, v) if v is _ else v for v in args_), *args_it)
+            pargs = (*(next(args_it, _) if v is _ or v is undefined else v for v in args_), *args_it)
         else:
             pargs = args_ + args
         kargs = {**kwargs_, **kwargs}
 
-        if _ in pargs or _ in kargs.values():
+        if _exist_placeholder(pargs) or _exist_placeholder(kargs.values()):
             return type(self)(self.func, *pargs, **kargs)
         return self.func(*pargs, **kargs)
 
@@ -44,7 +49,7 @@ class ppartial(partial):
     @classmethod
     def wrap(cls, func: Callable, /) -> Callable:
         def wrapper(*args, **kwargs):
-            if _ in args or _ in kwargs.items():
+            if _exist_placeholder(args) or _exist_placeholder(kwargs.values()):
                 return cls(func, *args, **kwargs)
             return func(*args, **kwargs)
         return update_wrapper(wrapper, func)
